package com.emato.cus.supervise.util;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.yaml.snakeyaml.Yaml;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

/**
 * 获取yml文件配置
 * 默认文件名为application.yml
 *
 * @author Scott Chen
 * 2019-08-22
 */
public class YmlUtils {
    private static final Logger logger = LoggerFactory.getLogger(YmlUtils.class);

    private static String ymlPath;

    private static final ThreadLocal<YmlUtils> ymlUtilsThreadLocal = new ThreadLocal<>();
    private static final ThreadLocal<Map<String, String>> ymlDataThreadLocal = new ThreadLocal<>();


    private YmlUtils(){}

    public static Builder builder() {
        return new Builder();
    }

    public static YmlUtils instance() {
        YmlUtils instance = ymlUtilsThreadLocal.get();
        if (instance == null) {
            instance = new YmlUtils();
            ymlUtilsThreadLocal.set(instance);
        }
        return instance;
    }


    public String getValue(String key) {
        return getValue(key, null);
    }

    /**
     * 根据key获取值
     *
     * @param key
     * @return
     */
    public String getValue(String key, String path) {
        Map<String, String> ymlData = ymlDataThreadLocal.get();
        if (ymlData == null){
            this.generateYmlData(path);
            ymlData = ymlDataThreadLocal.get();
        }
        return ymlData.get(key);
    }


    public YmlUtils generateYmlData(String path) {
        Map<String, String> ymlData = ymlDataThreadLocal.get();
        if (ymlData == null) {
            ymlData = getYmlDataByPath(path);
            ymlDataThreadLocal.set(ymlData);
        }
        return this;
    }

    /**
     * 根据文件名获取yml的文件内容
     *
     * @return
     */
    private static Map<String, String> getYmlDataByPath(String path) {
        Map<String, String> result = new HashMap<>();

        if (StringUtils.isBlank(path) && StringUtils.isBlank(ymlPath)) {
            String info = "yml配置文件路径为空";
            logger.error(info);
            throw new RuntimeException(info);
        }

        path = StringUtils.isBlank(path) ? ymlPath : path;

        InputStream in = null;

        try {
            in = YmlUtils.class.getClassLoader().getResourceAsStream(path);
        } catch (Exception e) {
            logger.error("读取 {} yml配置文件异常", path);
            e.printStackTrace();
        }

        Yaml props = new Yaml();
        Object obj = props.loadAs(in, Map.class);

        Map<String, Object> params = (Map<String, Object>) obj;

        for (Map.Entry<String, Object> entry : params.entrySet()) {
            String key = entry.getKey();
            Object val = entry.getValue();

            if (val instanceof Map) {
                forEachYaml(key, (Map<String, Object>) val, result);
            } else {
                if (val == null) {
                    continue;
                }
                result.put(key, val.toString().trim());
            }
        }
        return result;
    }

    /**
     * 遍历yml文件，获取map集合
     *
     * @param origin
     * @param obj
     * @return
     */
    public static Map<String, String> forEachYaml(String origin, Map<String, Object> obj, Map<String, String> result) {
        for (Map.Entry<String, Object> entry : obj.entrySet()) {
            String key = entry.getKey();
            Object val = entry.getValue();

            String newKey = "";
            if (StringUtils.isNoneBlank(origin)) {
                newKey = origin + "." + key;
            } else {
                newKey = key;
            }
            if (val instanceof Map) {
                forEachYaml(newKey, (Map<String, Object>) val, result);
            } else {
                result.put(newKey, val.toString());
            }
        }
        return result;
    }

    public static class Builder {
        private String ymlPath;

        public Builder() {
        }

        public Builder ymlPath(String ymlPath) {
            this.ymlPath = ymlPath;
            return this;
        }

        public YmlUtils build() {
            YmlUtils ymlUtils = new YmlUtils();
            YmlUtils.ymlPath = this.ymlPath;
            return ymlUtils;
        }
    }


    public static void main(String[] args) {
        String ymlPath = "application.yml";
        String key = "spring.app.name";
        System.out.println(YmlUtils.instance().getValue(key, ymlPath));
    }

}
